package com.glxp.api.admin.controller.inventory;

import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.annotation.AuthRuleAnnotation;
import com.glxp.api.admin.entity.inventory.InvWarehouseEntity;
import com.glxp.api.admin.req.basic.FilterBasicThirdSysRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inventory.FilterInvWarehouseRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.InvWarehouseResponse;
import com.glxp.api.admin.res.inventory.InvWarehouseThirdSysResponse;
import com.glxp.api.admin.service.inventory.InvWarehouseService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
public class InvWarehouseController {

    @Resource
    InvWarehouseService invWarehouseService;

    @GetMapping("spms/inv/warehouse/filter")
    public BaseResponse filterInvWarehouse(FilterInvWarehouseRequest filterInvWarehouseRequest) {
        List<InvWarehouseEntity> invWarehouseEntities = invWarehouseService.filterInvWarehouse(filterInvWarehouseRequest);
        List<InvWarehouseResponse> merge = merge(invWarehouseEntities, 0);

        Map<String, Object> restMap = new HashMap<>();
        restMap.put("list", merge);
        return ResultVOUtils.success(restMap);
    }

    @GetMapping("spms/inv/warehouse/filterAll")
    public BaseResponse filterAllInvWarehouse(FilterInvWarehouseRequest filterInvWarehouseRequest) {
        filterInvWarehouseRequest.setPid(0);
        List<InvWarehouseEntity> invWarehouseEntities = invWarehouseService.filterInvWarehouse(filterInvWarehouseRequest);
        return ResultVOUtils.success(invWarehouseEntities);
    }

    public List<InvWarehouseResponse> merge(List<InvWarehouseEntity> invWarehouseEntities,
                                            Integer pid) {
        List<InvWarehouseResponse> invWarehouseResponses = new ArrayList<>();
        for (InvWarehouseEntity invWarehouseEntity : invWarehouseEntities) {
            InvWarehouseResponse invWarehouseResponse = new InvWarehouseResponse();
            BeanUtils.copyProperties(invWarehouseEntity, invWarehouseResponse);
            if (pid.equals(invWarehouseEntity.getPid())) {
                invWarehouseResponse.setChildren(merge(invWarehouseEntities, invWarehouseEntity.getId()));
                invWarehouseResponses.add(invWarehouseResponse);
            }
        }
        return invWarehouseResponses;
    }


    @PostMapping("/spms/inv/warehouse/save")
    public BaseResponse save(@RequestBody @Valid InvWarehouseEntity invWarehouseEntity,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        if (invWarehouseEntity.getPid() == null) {
            invWarehouseEntity.setPid(0); // 默认设置
        }

        boolean b = invWarehouseService.insertInvWarehouse(invWarehouseEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("添加成功！");
    }

    @PostMapping("/spms/inv/warehouse/edit")
    public BaseResponse edit(@RequestBody @Valid InvWarehouseEntity invWarehouseEntity,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (invWarehouseEntity.getId() == null) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        invWarehouseEntity.setPid(null); // 不能修改父级 pid
        boolean b = invWarehouseService.updateInvWarehouse(invWarehouseEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success();
    }

    @PostMapping("/spms/inv/warehouse/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest) {

        if (deleteRequest.getId() == null) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        boolean b = invWarehouseService.deleteById(deleteRequest.getId());
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success();
    }

    /**
     * 绑定第三方仓库ID
     *
     * @param id
     * @param thridWarehouseId
     * @return
     */
    @AuthRuleAnnotation("")
    @PostMapping("/spms/inv/warehouse/bindThrWarehouse")
    public BaseResponse bindThrWarehouse(Integer id, Integer thridWarehouseId, String sysId) {
        if (null == id || null == thridWarehouseId || StrUtil.isBlank(sysId)) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        invWarehouseService.bindThrWarehouse(id, thridWarehouseId, sysId);
        return ResultVOUtils.success();
    }

    /**
     * 解绑第三方仓库ID
     *
     * @param id
     * @param thridWarehouseId
     * @return
     */
    @AuthRuleAnnotation("")
    @PostMapping("/spms/inv/warehouse/unbindThrWarehouse")
    public BaseResponse unbindThrWarehouse(Integer id, String sysId) {
        if (null == id || StrUtil.isBlank(sysId)) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        invWarehouseService.unbindThrWarehouse(id, sysId);
        return ResultVOUtils.success();
    }

    /**
     * 查询第三方系统和仓库数据关联信息
     *
     * @return
     */
    @AuthRuleAnnotation("")
    @GetMapping("/spms/inv/warehouse/thridSys/detail")
    public BaseResponse getThirdSysDetail(FilterInvWarehouseRequest request) {
        if (null == request || null == request.getId()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        FilterBasicThirdSysRequest sysRequest = new FilterBasicThirdSysRequest();
        sysRequest.setEnabled(true);
        List<InvWarehouseThirdSysResponse> list = invWarehouseService.getThirdSysDetail(sysRequest, request.getId());
        PageInfo<InvWarehouseThirdSysResponse> pageInfo;
        pageInfo = new PageInfo<>(list);
        PageSimpleResponse<InvWarehouseThirdSysResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

}
