package com.glxp.api.admin.controller.thrsys;

import com.glxp.api.admin.entity.thrsys.ThrInvWarehouseEntity;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.thrsys.FilterThrInvWarehouseRequest;
import com.glxp.api.admin.res.thrsys.ThrInvWarehouseResponse;
import com.glxp.api.admin.service.thrsys.ThrInvWarehouseService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 第三方仓库信息接口
 */
@Slf4j
@RestController
public class ThrInvWarehouseController {

    @Resource
    ThrInvWarehouseService thrInvWarehouseService;

    @GetMapping("spms/thrsys/warehouse/filter")
    public BaseResponse filterInvWarehouse(FilterThrInvWarehouseRequest filterThrInvWarehouseRequest) {
        List<ThrInvWarehouseEntity> thrInvWarehouseEntities = thrInvWarehouseService.filterThrInvWarehouse(filterThrInvWarehouseRequest);
        List<ThrInvWarehouseResponse> merge = merge(thrInvWarehouseEntities, 0);

        Map<String, Object> restMap = new HashMap<>();
        restMap.put("list", merge);
        return ResultVOUtils.success(restMap);
    }

    public List<ThrInvWarehouseResponse> merge(List<ThrInvWarehouseEntity> thrInvWarehouseEntities, Integer pid) {
        List<ThrInvWarehouseResponse> thrInvWarehouseResponses = new ArrayList<>();
        for (ThrInvWarehouseEntity thrInvWarehouseEntity : thrInvWarehouseEntities) {
            ThrInvWarehouseResponse thrInvWarehouseResponse = new ThrInvWarehouseResponse();
            BeanUtils.copyProperties(thrInvWarehouseEntity, thrInvWarehouseResponse);
            if (pid.equals(thrInvWarehouseEntity.getPid())) {
                thrInvWarehouseResponse.setChildren(merge(thrInvWarehouseEntities, thrInvWarehouseEntity.getId()));
                thrInvWarehouseResponses.add(thrInvWarehouseResponse);
            }
        }
        return thrInvWarehouseResponses;
    }

    @GetMapping("spms/thrsys/warehouse/filterAll")
    public BaseResponse filterAllInvWarehouse(FilterThrInvWarehouseRequest filterThrInvWarehouseRequest) {
        filterThrInvWarehouseRequest.setPid(0);
        List<ThrInvWarehouseEntity> thrInvWarehouseEntities = thrInvWarehouseService.filterThrInvWarehouse(filterThrInvWarehouseRequest);
        return ResultVOUtils.success(thrInvWarehouseEntities);
    }

    @PostMapping("/spms/thrsys/warehouse/save")
    public BaseResponse save(@RequestBody @Valid ThrInvWarehouseEntity thrInvWarehouseEntity,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        if (thrInvWarehouseEntity.getPid() == null) {
            thrInvWarehouseEntity.setPid(0); // 默认设置
        }
        boolean b = thrInvWarehouseService.insertInvWarehouse(thrInvWarehouseEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("添加成功！");
    }

    @PostMapping("/spms/thrsys/warehouse/edit")
    public BaseResponse edit(@RequestBody @Valid ThrInvWarehouseEntity thrInvWarehouseEntity,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (thrInvWarehouseEntity.getId() == null) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        thrInvWarehouseEntity.setPid(null); // 不能修改父级 pid
        boolean b = thrInvWarehouseService.updateInvWarehouse(thrInvWarehouseEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success();
    }

    @PostMapping("/spms/thrsys/warehouse/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest) {

        if (deleteRequest.getId() == null) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        boolean b = thrInvWarehouseService.deleteById(deleteRequest.getId());
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success();
    }

}
