package com.glxp.api.admin.service.business.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.business.StockOrderDao;
import com.glxp.api.admin.dao.business.StockOrderDetailDao;
import com.glxp.api.admin.entity.business.StockOrderDetailEntity;
import com.glxp.api.admin.entity.business.StockOrderEntity;
import com.glxp.api.admin.entity.business.StockOrderPrintEntity;
import com.glxp.api.admin.req.business.StockOrderDetailFilterRequest;
import com.glxp.api.admin.req.business.StockOrderFilterRequest;
import com.glxp.api.admin.req.business.StockOrderQueryRequest;
import com.glxp.api.admin.res.business.StockOrderDetailEntityVo;
import com.glxp.api.admin.res.business.StockOrderExportResponse;
import com.glxp.api.admin.service.business.StockOrderService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
public class StockOrderServiceImpl implements StockOrderService {

    @Resource
    StockOrderDao stockOrderDao;
    @Resource
    StockOrderDetailDao stockOrderDetailDao;

    @Override
    public List<StockOrderEntity> listStockOrders(StockOrderQueryRequest stockOrderQueryRequest) {
        if (stockOrderQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (stockOrderQueryRequest.getPage() - 1) * stockOrderQueryRequest.getLimit();
        PageHelper.offsetPage(offset, stockOrderQueryRequest.getLimit());
        return stockOrderDao.listStockOrder(stockOrderQueryRequest);
    }


    @Override
    public boolean insertStockOrder(StockOrderEntity stockOrderEntity) {
        return stockOrderDao.insertStockOrder(stockOrderEntity);
    }

    @Override
    public List<StockOrderEntity> listStockOrderNoDealWith(StockOrderQueryRequest stockOrderQueryRequest) {
        if (stockOrderQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (stockOrderQueryRequest.getPage() - 1) * stockOrderQueryRequest.getLimit();
        PageHelper.offsetPage(offset, stockOrderQueryRequest.getLimit());
        return stockOrderDao.listStockOrderNoDealWith(stockOrderQueryRequest);
    }

    @Override
    public boolean updateById(StockOrderEntity stockOrderEntity) {
        return stockOrderDao.updateById(stockOrderEntity);
    }

    @Override
    public boolean deleteByOrderId(String orderId) {
        return stockOrderDao.deleteByOrderId(orderId);
    }

    @Override
    public boolean deleteById(String id) {
        return stockOrderDao.deleteById(id);
    }

    @Override
    public boolean updateByStatus(String orderId, Integer status) {
        return stockOrderDao.updateByStatus(orderId, status);
    }

    @Override
    public List<StockOrderEntity> findAllOrders(StockOrderFilterRequest stockOrderFilterRequest) {
        return stockOrderDao.filterListOr(stockOrderFilterRequest);
    }

    @Override
    public List<StockOrderEntity> filterListOr(StockOrderFilterRequest stockOrderFilterRequest) {
        if (stockOrderFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (stockOrderFilterRequest.getPage() - 1) * stockOrderFilterRequest.getLimit();
        PageHelper.offsetPage(offset, stockOrderFilterRequest.getLimit());
        return stockOrderDao.filterListOr(stockOrderFilterRequest);
    }

    @Override
    public StockOrderEntity findOne(StockOrderFilterRequest stockOrderFilterRequest) {
        return stockOrderDao.findOne(stockOrderFilterRequest);
    }

    @Override
    public StockOrderEntity findById(String orderId) {
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        stockOrderFilterRequest.setId(orderId);
        return stockOrderDao.findOne(stockOrderFilterRequest);
    }

    @Override
    public StockOrderEntity findByBillNo(String orderId) {
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        stockOrderFilterRequest.setBillNo(orderId);
        return stockOrderDao.findOne(stockOrderFilterRequest);
    }


    @Override
    public List<StockOrderPrintEntity> listOrderPrint(String orderId) {
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        stockOrderFilterRequest.setId(orderId);
        return stockOrderDao.listOrderPrint(stockOrderFilterRequest);
    }

    @Override
    public List<StockOrderExportResponse> exportOrderData(StockOrderFilterRequest request) {
        //单据状态为已验收
        request.setStatus("5");
        List<StockOrderEntity> stockOrderEntities = stockOrderDao.filterListOr(request);
        if (CollUtil.isNotEmpty(stockOrderEntities)) {
            List<StockOrderExportResponse> result = new ArrayList<>();
            for (StockOrderEntity stockOrderEntity : stockOrderEntities) {
                StockOrderExportResponse response = new StockOrderExportResponse();
                BeanUtils.copyProperties(stockOrderEntity, response);
                StockOrderDetailFilterRequest filterRequest = new StockOrderDetailFilterRequest();
                filterRequest.setOrderIdFk(stockOrderEntity.getOrderIdFk());
                List<StockOrderDetailEntity> stockOrderDetailEntities = stockOrderDetailDao.filterStockOrderDetail(filterRequest);
                List<StockOrderDetailEntityVo> vos = BeanUtil.copyToList(stockOrderDetailEntities, StockOrderDetailEntityVo.class);
                response.setOrderDetails(vos);
                result.add(response);
            }
            return result;
        }
        return Collections.emptyList();
    }

}
