package com.glxp.api.admin.service.inventory.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ReflectUtil;
import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.basic.BasicThirdSysDao;
import com.glxp.api.admin.dao.inventory.InvWarehouseDao;
import com.glxp.api.admin.dao.thrsys.ThrInvWarehouseDao;
import com.glxp.api.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.api.admin.entity.inventory.InvWarehouseEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvWarehouseEntity;
import com.glxp.api.admin.req.basic.FilterBasicThirdSysRequest;
import com.glxp.api.admin.req.inventory.FilterInvWarehouseRequest;
import com.glxp.api.admin.res.inventory.InvWarehouseThirdSysResponse;
import com.glxp.api.admin.service.inventory.InvWarehouseService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
public class InvWarehouseServiceImpl implements InvWarehouseService {


    @Resource
    InvWarehouseDao invWarehouseDao;
    @Resource
    private BasicThirdSysDao basicThirdSysDao;
    @Resource
    private ThrInvWarehouseDao thrInvWarehouseDao;

    @Override
    public InvWarehouseEntity findDefault(Boolean advaceType, Boolean isDefault) {

        FilterInvWarehouseRequest filterInvWarehouseRequest = new FilterInvWarehouseRequest();
        filterInvWarehouseRequest.setIsDefault(isDefault);
        filterInvWarehouseRequest.setAdvanceType(advaceType);
        List<InvWarehouseEntity> invWarehouseEntities = invWarehouseDao.filterInvWarehouse(filterInvWarehouseRequest);
        if (invWarehouseEntities != null && invWarehouseEntities.size() > 0)
            return invWarehouseEntities.get(0);
        return null;
    }

    @Override
    public List<InvWarehouseEntity> filterInvWarehouse(FilterInvWarehouseRequest filterInvWarehouseRequest) {
        if (filterInvWarehouseRequest == null) {
            return Collections.emptyList();
        }
        if (filterInvWarehouseRequest.getPage() != null) {
            int offset = (filterInvWarehouseRequest.getPage() - 1) * filterInvWarehouseRequest.getLimit();
            PageHelper.offsetPage(offset, filterInvWarehouseRequest.getLimit());
        }
        List<InvWarehouseEntity> data = invWarehouseDao.filterInvWarehouse(filterInvWarehouseRequest);
        return data;
    }

    @Override
    public List<InvWarehouseEntity> filterGroupInvWarehouse(FilterInvWarehouseRequest filterInvWarehouseRequest) {
        if (filterInvWarehouseRequest == null) {
            return Collections.emptyList();
        }
        if (filterInvWarehouseRequest.getPage() != null) {
            int offset = (filterInvWarehouseRequest.getPage() - 1) * filterInvWarehouseRequest.getLimit();
            PageHelper.offsetPage(offset, filterInvWarehouseRequest.getLimit());
        }
        List<InvWarehouseEntity> data = invWarehouseDao.filterGroupInvWarehouse(filterInvWarehouseRequest);
        return data;
    }

    @Override
    public boolean insertInvWarehouse(InvWarehouseEntity invWarehouseEntity) {
        return invWarehouseDao.insertInvWarehouse(invWarehouseEntity);
    }

    @Override
    public boolean updateInvWarehouse(InvWarehouseEntity invWarehouseEntity) {
        return invWarehouseDao.updateInvWarehouse(invWarehouseEntity);
    }

    @Override
    public InvWarehouseEntity selectById(String id) {
        return invWarehouseDao.selectById(id);
    }

    @Override
    public InvWarehouseEntity selectByCode(String code) {
        FilterInvWarehouseRequest filterInvWarehouseRequest = new FilterInvWarehouseRequest();
        filterInvWarehouseRequest.setCode(code);
        List<InvWarehouseEntity> invWarehouseEntities = invWarehouseDao.filterInvWarehouse(filterInvWarehouseRequest);
        if (invWarehouseEntities != null && invWarehouseEntities.size() > 0)
            return invWarehouseEntities.get(0);
        return null;
    }

    @Override
    public boolean deleteById(String id) {
        return invWarehouseDao.deleteById(id);
    }

    @Override
    public boolean bindThrWarehouse(Integer id, Integer thridWarehouseId, String sysId) {
        return invWarehouseDao.updateThridId(id, thridWarehouseId, sysId);
    }

    @Override
    public boolean unbindThrWarehouse(Integer id,String sysId) {
        return invWarehouseDao.updateThridId(id, null, sysId);
    }

    @Override
    public List<InvWarehouseThirdSysResponse> getThirdSysDetail(FilterBasicThirdSysRequest request, Integer id) {
        //查询第三方系统数据
        if (request == null) {
            return Collections.emptyList();
        }
        if (request.getPage() != null) {
            int offset = (request.getPage() - 1) * request.getLimit();
            PageHelper.offsetPage(offset, request.getLimit());
        }
        List<BasicThirdSysEntity> data = basicThirdSysDao.filterBasicThiSys(request);
        if (CollUtil.isEmpty(data)) {
            return Collections.emptyList();
        }
        List<InvWarehouseThirdSysResponse> result = new ArrayList<>(data.size());
        InvWarehouseEntity invWarehouseEntity = invWarehouseDao.selectById(String.valueOf(id));
        for (BasicThirdSysEntity sys : data) {
            if (sys.getThirdId().equals("thirdId")) {
                InvWarehouseThirdSysResponse response = new InvWarehouseThirdSysResponse();
                response.setSysId(sys.getThirdId());
                response.setSysName(sys.getThirdName());
                Object thirdId = ReflectUtil.getFieldValue(invWarehouseEntity, sys.getThirdId());
                if (null != thirdId) {
                    ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectByCode(String.valueOf(thirdId));
                    response.setThirdId(thrInvWarehouseEntity.getCode());
                    response.setThirdName(thrInvWarehouseEntity.getName());
                }
                result.add(response);
            } else if (sys.getThirdId().equals("thirdId1")) {
                InvWarehouseThirdSysResponse response = new InvWarehouseThirdSysResponse();
                response.setSysId(sys.getThirdId());
                response.setSysName(sys.getThirdName());
                Object thirdId = ReflectUtil.getFieldValue(invWarehouseEntity, sys.getThirdId());
                if (null != thirdId) {
                    ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectByCode(String.valueOf(thirdId));
                    response.setThirdId(thrInvWarehouseEntity.getCode());
                    response.setThirdName(thrInvWarehouseEntity.getName());
                }
                result.add(response);
            }else if (sys.getThirdId().equals("thirdId2")) {
                InvWarehouseThirdSysResponse response = new InvWarehouseThirdSysResponse();
                response.setSysId(sys.getThirdId());
                response.setSysName(sys.getThirdName());
                Object thirdId = ReflectUtil.getFieldValue(invWarehouseEntity, sys.getThirdId());
                if (null != thirdId) {
                    ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectByCode(String.valueOf(thirdId));
                    response.setThirdId(thrInvWarehouseEntity.getCode());
                    response.setThirdName(thrInvWarehouseEntity.getName());
                }
                result.add(response);
            }else if (sys.getThirdId().equals("thirdId3")) {
                InvWarehouseThirdSysResponse response = new InvWarehouseThirdSysResponse();
                response.setSysId(sys.getThirdId());
                response.setSysName(sys.getThirdName());
                Object thirdId = ReflectUtil.getFieldValue(invWarehouseEntity, sys.getThirdId());
                if (null != thirdId) {
                    ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectByCode(String.valueOf(thirdId));
                    response.setThirdId(thrInvWarehouseEntity.getCode());
                    response.setThirdName(thrInvWarehouseEntity.getName());
                }
                result.add(response);
            }else if (sys.getThirdId().equals("thirdId4")) {
                InvWarehouseThirdSysResponse response = new InvWarehouseThirdSysResponse();
                response.setSysId(sys.getThirdId());
                response.setSysName(sys.getThirdName());
                Object thirdId = ReflectUtil.getFieldValue(invWarehouseEntity, sys.getThirdId());
                if (null != thirdId) {
                    ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectByCode(String.valueOf(thirdId));
                    response.setThirdId(thrInvWarehouseEntity.getCode());
                    response.setThirdName(thrInvWarehouseEntity.getName());
                }
                result.add(response);
            }
        }
        return result;
    }

}
